import json
import sys

from napalm import get_network_driver
from nornir.core.task import Result, Task
from tabulate import tabulate
from tqdm import tqdm

try:
    from .c_nornir import DevopsNornir
except Exception:
    from c_nornir import DevopsNornir


try:
    from .c_login import Login
except Exception:
    from c_login import Login


class Alive:

    def __init__(self, path: str = None, filter: str = None, workers: int = 2):
        nornir = DevopsNornir(path=path, filter=filter, workers=workers)
        self.nr = nornir.init()
        self.n_devices = len(self.nr.inventory.hosts.keys())
        try:
            self.equitativo = int(100 / int(self.n_devices))
        except Exception:
            self.equitativo = 100
        self.up = self.equitativo if self.equitativo % 2 == 0 else self.equitativo + 1  # noqa
        self.indent = 3
        self.sort_keys = True
        self.data = None

        login = Login()
        if login.validateToken() is False:
            raise Exception("Login failed")

    def update_pbar(self):
        self.pbar.update(self.up)
        self.pbar.refresh()
        sys.stdout.flush()

    def data_json(self):
        data = json.dumps(self.data, indent=self.indent,
                          sort_keys=self.sort_keys)
        return data

    def data_table(self):
        array_for_tabulate = [['name', 'is_alive']]
        for item in self.data:
            try:
                array_for_tabulate.append(
                    [item, self.data[item]['data']['is_alive']])
            except Exception:
                pass
        data = (tabulate(array_for_tabulate, headers='firstrow', tablefmt='grid'))  # noqa
        return data

    def alive(self, task: Task):
        try:
            driver = get_network_driver(task.host.platform)

            device = driver(
                hostname=task.host.hostname,
                username=task.host.username,
                password=task.host.password,
                optional_args={'port': task.host.port},
            )

            device.open()

            result = Result(
                host=task.host,
                result=device.is_alive()
            )

            device.close()

        except Exception:
            result = {}

        self.update_pbar()

        return result

    def run(self):
        dict_result = {}

        # solo si hay dispositivos en el inventario
        if self.n_devices > 0:
            self.pbar = tqdm(total=100)

            result = self.nr.run(
                name="alive",
                task=self.alive,
            )

            self.pbar.close()

            for item in result:
                dict_result[item] = {"data": result[item][0].result}

        self.data = dict_result

        return dict_result



from nornir import InitNornir
from nornir.core.filter import F


class DevopsNornir:

    def __init__(self, path: str = None, filter: str = None, workers: int = 2):
        self.nr = InitNornir(
            logging={"log_file": "ciberc-ca.log", "level": "DEBUG"},

            runner={
                "plugin": "threaded",
                "options": {
                    "num_workers": workers,
                },
            },
            inventory={
                "plugin": 'SimpleInventory',

                "options": {
                    "host_file": f"{path}/hosts.yaml",
                    "group_file": f"{path}/groups.yaml",
                    "defaults_file": f"{path}/defaults.yaml",
                }
            },
            dry_run=True
        )

        if filter is not None:
            for group in filter.split(","):
                self.nr = self.nr.filter(F(groups__contains=group))

    def init(self):
        return self.nr



import hvac
import yaml


class Login():

    def __init__(self):
        self.token = None
        self.url = 'http://206.189.175.222:8200'
        self.client = hvac.Client(url=self.url)

    def init(self, username, password):

        params = {
            'username': username,
            'password': password
        }
        try:
            gen_token = self.client.login(
            url=f'v1/auth/userpass/login/{username}', use_token=True, json=params)  # noqa
            self.token = gen_token['auth']['client_token']
        except Exception:
            pass

    def saveToken(self):
        # -- almacenando token
        params = [{'token': self.token}]
        with open(r'credentials.yaml', 'w') as file:
            yaml.dump(params, file)

    def getToken(self):
        try:
            with open(r'credentials.yaml') as file:
                documents = yaml.full_load(file)[0]
            return documents['token']
        except Exception:
            return None

    def validateToken(self):
        self.token = self.getToken()

        if self.token is None:
            return False

        token_validate = hvac.Client(url=self.url, token=self.token)
        while token_validate.is_authenticated():
            return True
        else:
            return False



import json
import sys

from napalm import get_network_driver
from nornir.core.task import Result, Task
from tqdm import tqdm
from ttp import ttp

try:
    from .c_nornir import DevopsNornir
except Exception:
    from c_nornir import DevopsNornir

try:
    from .c_templates import StringTemplates
except Exception:
    from c_templates import StringTemplates

try:
    from .c_interfaces_excel import ExcelInterfaces
except Exception:
    from c_interfaces_excel import ExcelInterfaces

try:
    from .c_login import Login
except Exception:
    from c_login import Login


class Interfaces:

    def __init__(self, path: str = None, filter: str = None, workers: int = 1):
        nornir = DevopsNornir(path=path, filter=filter, workers=workers)
        self.nr = nornir.init()
        self.n_devices = len(self.nr.inventory.hosts.keys())
        try:
            self.equitativo = int(100 / int(self.n_devices))
        except Exception:
            self.equitativo = 100
        self.up = self.equitativo if self.equitativo % 2 == 0 else self.equitativo + 1  # noqa
        self.indent = 3
        self.sort_keys = True
        self.data = None
        self.template = StringTemplates()
        self.exclude = -1

        login = Login()
        if login.validateToken() is False:
            raise Exception("Login failed")

    def update_pbar(self):
        self.pbar.update(self.up)
        self.pbar.refresh()
        sys.stdout.flush()

    def data_json(self):
        data = json.dumps(self.data, indent=self.indent,
                          sort_keys=self.sort_keys)
        return data

    def data_excel(self, mechanism, name):
        try:
            excel = ExcelInterfaces()
            excel.create(self.data, mechanism)
            excel.save(name)
        except Exception:
            return False

        return True

    def get_interfaces(self, device):
        dict_interfaces = device.get_interfaces()
        filtrado = {}

        if len(dict_interfaces) > 0:
            # filtrado de interfaces solo activas
            filtrado = dict(
                filter(lambda i: i[1]['is_up'] is True, dict_interfaces.items()))  # noqa

            filtrado = dict(filter(
                lambda i: i[0].find('Vlan') == self.exclude
                and i[0].find('Loop') == self.exclude
                and i[0].find('Tu') == self.exclude, filtrado.items()
            ))

        return filtrado

    def get_services(self, device, interfaces_filtered):
        data = interfaces_filtered.copy()

        if len(interfaces_filtered) > 0:
            for key, value in interfaces_filtered.items():
                comando = f"show run interface {key}"
                interface_data = device.cli([comando])

                try:
                    string_services = interface_data[comando]
                    parser = ttp(data=string_services,
                                 template=self.template.get_ttp_services())
                    parser.parse()
                except Exception:
                    pass

                try:
                    services_per_interface = parser.result()[0][0]['services']
                except Exception:
                    # si la interface es modo trocal
                    # get_ttp_trunk
                    try:
                        string_services = interface_data[comando]
                        parser = ttp(data=string_services,
                                     template=self.template.get_ttp_trunk())
                        parser.parse()
                    except Exception:
                        pass

                    try:
                        services_per_interface = parser.result()[0][0]['services']  # noqa
                    except Exception:
                        services_per_interface = {}

                data[key]['services'] = services_per_interface

        return data

    def get_vlan(self, device, data):

        for item in data:
            # recorre por cada interface los servicios
            try:
                services_list = [data[item]["services"]] if data[item]["services"].__class__ == dict else data[item]["services"]  # noqa
            except Exception:
                services_list = []

            if len(services_list) != 0:
                for service in services_list:
                    # validar si es trunk
                    # service_instance en el item
                    if 'service_instance' in service:

                        # verificar vlans para las interfaces troncales

                        try:

                            vlans_list = service['data']['switchport_vlans']
                            service['vlan'] = list([])

                            for vl in vlans_list:
                                try:
                                    comando_vlan = f"show run interface vlan {vl}"  # noqa
                                    dato_vlan = device.cli([comando_vlan])
                                    string_vlan = dato_vlan[comando_vlan]

                                    parser_vlan = ttp(data=string_vlan, template=self.template.get_ttp_vlan())  # noqa
                                    parser_vlan.parse()
                                except Exception:
                                    # el servicio no tiene vlans configuradas
                                    pass

                                try:
                                    vlan_per_servicio = parser_vlan.result()[0][0]['interface_vlan']  # noqa
                                except Exception:
                                    vlan_per_servicio = {}

                                service['vlan'].append(vlan_per_servicio)  # noqa

                        except Exception:
                            # interfaces que poseen servicios

                            try:
                                comando_vlan = f"show run interface vlan {service['service_instance']}"  # noqa
                                dato_vlan = device.cli([comando_vlan])
                                string_vlan = dato_vlan[comando_vlan]

                                parser_vlan = ttp(data=string_vlan, template=self.template.get_ttp_vlan())  # noqa
                                parser_vlan.parse()
                            except Exception:
                                # el servicio no tiene vlans configuradas
                                pass

                            try:
                                vlan_per_servicio = parser_vlan.result()[0][0]['interface_vlan']  # noqa
                            except Exception:
                                vlan_per_servicio = {}

                            service['vlan'] = [vlan_per_servicio]  # noqa

        return data

    def get_validacion_vlans(self, device, data):

        for item in data:
            # recorre por cada interface los servicios
            try:
                services_list = [data[item]["services"]] if data[item]["services"].__class__ == dict else data[item]["services"]  # noqa
            except Exception:
                services_list = []

            # se recorren los servicios por cada interface
            for service in services_list:
                try:
                    vlans_list = [service['vlan']] if service['vlan'].__class__ == dict else service['vlan']  # noqa
                except Exception:
                    vlans_list = []

                # ser recorren las vlans de cada servicio
                for vlan in vlans_list:
                    # bridge domain
                    bridge_domain = service["service_instance"]

                    # --
                    # -- primer comando para optener las interfaces bridge
                    # --
                    try:
                        comando_bridge = f"show bridge-domain {bridge_domain}"  # noqa
                        int_valn = device.cli([comando_bridge])
                        string_vlan = int_valn[comando_bridge]

                        parse = ttp(data=string_vlan, template=self.template.get_ttp_bridge())  # noqa
                        parse.parse()
                    except Exception:
                        pass

                    try:
                        int_bridges = parse.result()[0][0]['bridges']  # noqa
                    except Exception:
                        int_bridges = {}

                    # asignacion de las interfaces bridge_domain a la vlan
                    vlan['bridges'] = int_bridges

                    cant_bridges = 0
                    if len(int_bridges) > 0:
                        cant_bridges = len(
                            int_bridges['interfaces']) if int_bridges['interfaces'].__class__ == list else 1 if int_bridges != {} else 0  # noqa
                    vlan['cant_bridges'] = cant_bridges

                    # --
                    # -- segundo comando para optener los puertos
                    # --
                    try:
                        comando_bridge = f"show mac-address-table dynamic vlan {bridge_domain}"  # noqa
                        ports_vlan = device.cli([comando_bridge])
                        string_vlan = ports_vlan[comando_bridge]

                        parse_dyn = ttp(data=string_vlan, template=self.template.get_ttp_ports())  # noqa
                        parse_dyn.parse()
                    except Exception:
                        pass

                    try:
                        ports = parse_dyn.result()[0][0]['ports']  # noqa
                    except Exception:
                        ports = {}

                    # asignacion de las puertos de la vlan
                    vlan['ports'] = ports
                    vlan['cant_ports'] = len(ports) if ports.__class__ == list else 1 if ports != {} else 0  # noqa

        return data

    def execute(self, task: Task):
        try:
            driver = get_network_driver(task.host.platform)

            device = driver(
                hostname=task.host.hostname,
                username=task.host.username,
                password=task.host.password,
                timeout=10,
                optional_args={'port': task.host.port},
            )

            # [paso 01] apertura de la unica sesion por equipo
            device.open()

            # [paso 02] recolectar todas las interfaces del equipo
            datos_interfaces = self.get_interfaces(device)

            # [paso 03] recolectar todas las interfaces del equipo los servicios # noqa
            datos_interfaces = self.get_services(device, datos_interfaces)

            # [paso 04] recolectar todas las interfaces del equipo por servicio las vlan # noqa
            datos_interfaces = self.get_vlan(device, datos_interfaces)

            # [paso 05] recoletar por cada vlan de los servicios
            # las validaciones correspondientes
            datos_interfaces = self.get_validacion_vlans(device, datos_interfaces)  # noqa

            # validar conexion del dispositivo
            result = Result(
                host=task.host,
                result=datos_interfaces
            )

            device.close()

        except Exception as e:
            try:
                device.close()
            except Exception:
                pass

            result = Result(
                host=task.host,
                result={"error": str(e)}
            )

        self.update_pbar()

        return result

    def run(self):
        dict_result = {}

        # solo si hay dispositivos en el inventario
        if self.n_devices > 0:
            self.pbar = tqdm(total=100)

            result = self.nr.run(
                name="interfaces",
                task=self.execute,
            )

            self.pbar.close()

            for item in result:
                try:
                    data = result[item][0].result
                except Exception:
                    data = {}
                dict_result[item] = data

        self.data = dict_result

        return dict_result



import os
from typing import Optional

try:
    from .c_typer import app, typer
except Exception:
    from c_typer import app, typer  # noqa

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

try:
    from .c_alive import Alive
except Exception:
    from c_alive import Alive


@app.command()
def alive(
    path: Optional[str] = None,
    group: Optional[str] = None,
    workers: Optional[int] = None,
    output: Optional[str] = None,
):
    """
    alive for all device filter with groups
    """

    if path is None:
        path_inventory = BASE_DIR + "/inventory/"
    else:
        path_inventory = path

    alive = Alive(path=path_inventory, filter=group, workers=workers)
    alive.run()

    if output == 'json':
        print(alive.data_json())

    if output == 'table':
        print(alive.data_table())



import os

try:
    from .c_typer import app, typer
except Exception:
    from c_typer import app, typer

BASE_DIR = os.path.dirname(os.path.abspath(__file__))


@app.command()
def inventory(
    create: bool = typer.Option(
        "",
        help="create files from inventory examples",
    ),
):
    """
    create files for inventory system
    """
    if create:
        exists = os.path.exists("./inventory/")
        if not exists:
            os.system(f"cp -r {BASE_DIR}/inventory/ .")
        else:
            print("inventory already exists")



ttp_template_services = """
<group name="services">
 service instance {{service_instance}} {{type_instance}}
  <group name="data">
  description {{description | ORPHRASE}}
  </group>
  <group name="data">
  encapsulation dot1q {{dot1q}}
  </group>
  <group name="data">
  rewrite ingress tag pop 1 {{rewrite_ingress}}
  </group>
  <group name="data">
  xconnect {{xconnect_ip}} {{xconnect_id}} {{xconnect_class}} {{xconnect_class_data}}
  </group>
  <group name="data">
  service-policy input {{service_policy_input}}
  </group>
  <group name="data">
  service-policy output {{service_policy_output}}
  </group>
  <group name="data">
  bridge-domain {{bridge_domain}}
  </group>
</group>
"""  # noqa


ttp_template_vlan_service = """
<group name="interface_vlan">
interface {{interface_vlan}}
 <group name="description">
 description {{description | ORPHRASE}}
 </group>
 <group name="vrf">
 ip vrf forwarding {{vrf}}
 </group>
 <group name="ip_address_primary">
 ip address {{ip_address}} {{mask}}
 </group>
 <group name="ip_address_secundary">
 ip address {{ip_address}} {{mask}} {{type}}
 </group>
 <group name="ip_access_group">
 ip access-group {{access_group}} {{type}}
 </group>
 <group name="service_policy_input">
 service-policy input {{service_policy_input}}
 </group>
 <group name="service_policy_output">
 service-policy output {{service_policy_output}}
 </group>
 <group name="ip_helper_address">
 ip helper-address {{ip_helper_address}}
 </group>
 <group name="state">
 {{state}}
 </group>
</group>
"""  # noqa


ttp_template_bridge = """
<group name="bridges">
State: {{state}}                    Mac learning: {{mac_learning}}
    <group name="interfaces">
    {{interface}} {{type}} {{ins}} {{bridge_domain}}
    </group>
</group>
"""  # noqa

ttp_template_ports = """
<group name="ports">
{{num}}  {{vlan}}  {{mac_address}}   {{type}}  {{learn}}          {{age}}   {{port | ORPHRASE}}
</group>
"""  # noqa


ttp_template_trunk = """
<group name="services">
 <group name="data">
 switchport trunk encapsulation {{dot1q}}
 </group>
 <group name="data">
 switchport trunk allowed vlan {{switchport_vlans | | split(',')}}
 </group>
 <group name="data">
 switchport mode {{switchport_mode}}
 </group>
 <group name="data">
 mls {{mls}}
 </group>
</group>
"""


class StringTemplates:

    def get_ttp_services(self):
        global ttp_template_services
        return ttp_template_services

    def get_ttp_trunk(self):
        global ttp_template_trunk
        return ttp_template_trunk

    def get_ttp_vlan(self):
        global ttp_template_vlan_service
        return ttp_template_vlan_service

    def get_ttp_bridge(self):
        global ttp_template_bridge
        return ttp_template_bridge

    def get_ttp_ports(self):
        global ttp_template_ports
        return ttp_template_ports



from openpyxl import Workbook
from openpyxl.styles import Font, PatternFill

try:
    from .c_login import Login
except Exception:
    from c_login import Login


class ExcelInterfaces():

    def __init__(self):
        self.wb = Workbook()
        self.ws = self.wb.create_sheet("interfaces", 0)
        self.font_header = Font(color='ffffff', bold=True)

        login = Login()
        if login.validateToken() is False:
            raise Exception("Login failed")

    def setHeadCol(self, ws, col, name):
        self.ws[col] = name
        self.ws[col].fill = PatternFill(
            start_color='000000', end_color='000000', fill_type="solid")
        self.ws[col].font = self.font_header
        return ws

    def setHeadColInterface(self, ws, col, name):
        self.ws[col] = name
        self.ws[col].fill = PatternFill(
            start_color='eccd6a', end_color='eccd6a', fill_type="solid")
        self.ws[col].font = self.font_header
        return ws

    def getvalueFroomDictInterface(self, data, device, interface, name):
        value = None
        try:
            value = data[device][interface][name]
        except Exception:
            pass

        return value

    def getValueFromArrayService(self, service, name):
        valor = None
        try:
            for item in service['data']:
                try:
                    valor = (item[name])
                except Exception:
                    pass
        except Exception:
            pass

        return valor

    def create(self, data, mechanism: str = "row"):

        self.setHeadCol(self.ws, 'A1',  "Interfaz antigua")
        self.setHeadCol(self.ws, 'B1', "Interfaz nueva")
        self.setHeadCol(self.ws, 'C1', "Negotation / STP")
        self.setHeadCol(self.ws, 'D1', "BVI")
        self.setHeadCol(self.ws, 'E1', "BD")  # bridge_domain
        self.setHeadCol(self.ws, 'F1', "DOT1Q")
        self.setHeadCol(self.ws, 'G1', "DESCRIPTION")
        self.setHeadCol(self.ws, 'H1', "MTU")
        self.setHeadCol(self.ws, 'I1', "VRF")
        self.setHeadCol(self.ws, 'J1', "XCONNECTION")
        self.setHeadCol(self.ws, 'K1', "XCONNECTION VC ID")
        self.setHeadCol(self.ws, 'L1', "PW-CLASS")
        self.setHeadCol(self.ws, 'M1', "QoS INPUT")
        self.setHeadCol(self.ws, 'N1', "QoS OUTPUT")
        self.setHeadCol(self.ws, 'O1', "IP ADDRESS")

        if mechanism != "column":
            # primary, secundary
            self.setHeadCol(self.ws, 'P1', "TYPE IP ADDRESS")

        # incicio de fila en el excel
        index = 2

        # --
        # -- PRIMERA iteracion son los resultados en los device (hosts)
        # --
        for device in data:

            EC_NAME_DEVICE = device  # noqa

            # --
            # -- SEGUNDA iteracion con las interfaces
            # --
            for interface in data[device]:

                EC_INTERFACE_NAME = interface
                EC_INTERFACE_DESCRIPTION = self.getvalueFroomDictInterface(data, device, interface, 'description')  # noqa
                EC_INTERFACE_MTU = self.getvalueFroomDictInterface(data, device, interface, 'mtu')  # noqa

                if EC_INTERFACE_NAME == "error":
                    break

                # primera iteracion con excel va solo la interface
                self.ws[f"A{index}"] = EC_INTERFACE_NAME
                self.ws[f"G{index}"] = EC_INTERFACE_DESCRIPTION
                self.ws[f"H{index}"] = EC_INTERFACE_MTU
                index += 1

                # --
                # -- TERCER iteracion, recorre los servicios por interface
                # --
                try:
                    services_list = [data[device][interface]["services"]] if data[device][interface]["services"].__class__ == dict else data[device][interface]["services"]  # noqa
                except Exception:
                    services_list = []

                if services_list != [{}]:  # noqa
                    for service in services_list:
                        EC_SERVICE_BRIDGE_DOMANIN = self.getValueFromArrayService(service, 'bridge_domain')  # noqa
                        EC_SERVICE_DOT1Q = self.getValueFromArrayService(service, 'dot1q')  # noqa
                        EC_SERVICE_QOS_INPUT = self.getValueFromArrayService(service, 'service_policy_input')  # noqa
                        EC_SERVICE_QOS_OUTPUT = self.getValueFromArrayService(service, 'service_policy_output')  # noqa

                        EC_SERVICE_XCONNECTIONS = self.getValueFromArrayService(service, 'xconnect_ip')  # noqa
                        EC_SERVICE_XCONNECTIONS_VC_ID = self.getValueFromArrayService(service, 'xconnect_id')  # noqa
                        EC_SERVICE_PW_CLASS = self.getValueFromArrayService(service, 'xconnect_class')  # noqa
                        EC_SERVICE_PW_CLASS_DATA = self.getValueFromArrayService(service, 'xconnect_class_data')  # noqa

                        EC_SERVICE_DESCRIPTION = self.getValueFromArrayService(service, "description")  # noqa
                        EC_SERVICE_MODE_TRUNK = self.getValueFromArrayService(service, "switchport_mode")  # noqa

                        # --
                        # -- CUARTA iteracion, recorre los servicios por interface # noqa: E501
                        # --

                        try:
                            vlans_list = [service['vlan']] if service['vlan'].__class__ == dict else service['vlan']  # noqa
                        except Exception:
                            vlans_list = []

                        if len(vlans_list) > 0 and vlans_list != [{}]:
                            for vlan in vlans_list:
                                # NOTA:
                                # service_policy_input, service_policy_output (son del servicio) # noqa
                                # si se encuentra en la vlan se omite, debe ser del servicio # noqa
                                # ip_helper_address => va como nota en el excel

                                try:
                                    EC_VLAN_DESCRIPTION = vlan['description']['description']  # noqa
                                except Exception:
                                    EC_VLAN_DESCRIPTION = None

                                try:
                                    EC_VLAN_VRF = vlan['vrf']['vrf']  # noqa
                                except Exception:
                                    EC_VLAN_VRF = None  # noqa

                                try:
                                    EC_VLAN_QOS_INPUT = vlan['service_policy_input']['service_policy_input']  # noqa
                                except Exception:
                                    EC_VLAN_QOS_INPUT = None  # noqa

                                try:
                                    EC_VLAN_QOS_OUTPUT = vlan['service_policy_output']['service_policy_output']  # noqa
                                except Exception:
                                    EC_VLAN_QOS_OUTPUT = None  # noqa

                                try:
                                    EC_VLAN_IP_ADDRESS_PRIMARY = vlan['ip_address_primary']['ip_address']  # noqa
                                    EC_VLAN_IP_ADDRESS_MASK = vlan['ip_address_primary']['mask']  # noqa

                                    if EC_VLAN_IP_ADDRESS_PRIMARY is not None:
                                        EC_VLAN_PRIMARY_IP = f"{EC_VLAN_IP_ADDRESS_PRIMARY} {EC_VLAN_IP_ADDRESS_MASK}"  # noqa
                                        EC_VLAN_IP_ADDRESS_TYPE = 'primary'
                                except Exception:
                                    EC_VLAN_PRIMARY_IP = None
                                    EC_VLAN_IP_ADDRESS_TYPE = None

                                # --
                                # -- [VALIDACION] - saber si el servicio es BVI
                                EC_BVI = None
                                IS_BVI = "BVI"
                                try:
                                    # los troncales no pueden ser BVI
                                    if EC_SERVICE_MODE_TRUNK is not None:
                                        try:
                                            cant_ports = vlan['cant_ports']
                                            cant_bridges = vlan['cant_bridges']
                                        except Exception:
                                            cant_ports = 0
                                            cant_bridges = 0

                                        # es BVI si tiene mas de dos puertos, con mas de dos bridges # noqa
                                        if cant_ports > 0 and cant_bridges > 1:
                                            EC_BVI = IS_BVI

                                        # si el servicio es de gestion debe tener mas de dos bridges # noqa
                                        # para poder ser BVI
                                        elif EC_VLAN_DESCRIPTION.lower().find("gest") and cant_bridges > 1:  # noqa
                                            EC_BVI = IS_BVI

                                        # sera BVI si es capa2 con mas de dos bridges # noqa
                                        elif EC_SERVICE_XCONNECTIONS is not None and cant_bridges > 1:  # noqa
                                            EC_BVI = IS_BVI

                                        # si todas las validaciones no se encuentran y existen # noqa
                                        # mas de dos puertos en la mac_address table es BVI # noqa
                                        elif cant_ports > 1:
                                            EC_BVI = IS_BVI

                                except Exception:
                                    pass

                                self.ws[f"A{index}"] = EC_INTERFACE_NAME
                                self.ws[f"D{index}"] = EC_BVI
                                self.ws[f"E{index}"] = EC_SERVICE_BRIDGE_DOMANIN  # noqa
                                self.ws[f"F{index}"] = EC_SERVICE_DOT1Q
                                self.ws[f"G{index}"] = EC_VLAN_DESCRIPTION

                                self.ws[f"I{index}"] = EC_VLAN_VRF
                                self.ws[f"J{index}"] = EC_SERVICE_XCONNECTIONS
                                self.ws[f"K{index}"] = EC_SERVICE_XCONNECTIONS_VC_ID  # noqa
                                self.ws[f"L{index}"] = EC_SERVICE_PW_CLASS_DATA if EC_SERVICE_PW_CLASS == 'pw_class' else None  # noqa

                                self.ws[f"M{index}"] = EC_SERVICE_QOS_INPUT  # if EC_VLAN_QOS_INPUT == None else EC_VLAN_QOS_INPUT  # noqa
                                self.ws[f"N{index}"] = EC_SERVICE_QOS_OUTPUT  # if EC_VLAN_QOS_OUTPUT == None else EC_VLAN_QOS_OUTPUT  # noqa
                                self.ws[f"O{index}"] = EC_VLAN_PRIMARY_IP

                                if mechanism == 'row':
                                    self.ws[f"P{index}"] = EC_VLAN_IP_ADDRESS_TYPE  # noqa
                                    index += 1

                                    try:
                                        ip_secundary_list = [vlan['ip_address_secundary']] if vlan['ip_address_secundary'].__class__ == dict else vlan['ip_address_secundary']  # noqa
                                    except Exception:
                                        ip_secundary_list = []

                                    if ip_secundary_list is not [{}]:
                                        for secundary in ip_secundary_list:

                                            try:
                                                EC_VLAN_IP_ADDRESS = secundary['ip_address']  # noqa
                                                EC_VLAN_IP_ADDRESS_MASK = secundary['mask']  # noqa
                                                EC_VLAN_IP_ADDRESS_TYPE = secundary['type']  # noqa

                                                if EC_VLAN_IP_ADDRESS is not None:  # noqa
                                                    EC_VLAN_IP_SECUNDARY = f"{EC_VLAN_IP_ADDRESS} {EC_VLAN_IP_ADDRESS_MASK}"  # noqa
                                            except Exception:
                                                EC_VLAN_IP_SECUNDARY = None

                                            self.ws[f"A{index}"] = EC_INTERFACE_NAME  # noqa
                                            self.ws[f"D{index}"] = EC_BVI  # noqa
                                            self.ws[f"E{index}"] = EC_SERVICE_BRIDGE_DOMANIN  # noqa
                                            self.ws[f"F{index}"] = EC_SERVICE_DOT1Q  # noqa
                                            self.ws[f"G{index}"] = EC_VLAN_DESCRIPTION  # noqa

                                            self.ws[f"I{index}"] = EC_VLAN_VRF  # noqa
                                            self.ws[f"J{index}"] = EC_SERVICE_XCONNECTIONS  # noqa
                                            self.ws[f"K{index}"] = EC_SERVICE_XCONNECTIONS_VC_ID  # noqa
                                            self.ws[f"L{index}"] = EC_SERVICE_PW_CLASS_DATA if EC_SERVICE_PW_CLASS == 'pw_class' else None  # noqa

                                            self.ws[f"M{index}"] = EC_SERVICE_QOS_INPUT  # if EC_VLAN_QOS_INPUT == None else EC_VLAN_QOS_INPUT  # noqa
                                            self.ws[f"N{index}"] = EC_SERVICE_QOS_OUTPUT  # if EC_VLAN_QOS_OUTPUT == None else EC_VLAN_QOS_OUTPUT  # noqa
                                            self.ws[f"O{index}"] = EC_VLAN_IP_SECUNDARY  # noqa
                                            self.ws[f"P{index}"] = EC_VLAN_IP_ADDRESS_TYPE  # noqa
                                            index += 1

                                else:

                                    try:
                                        ip_secundary_list = [vlan['ip_address_secundary']] if vlan['ip_address_secundary'].__class__ == dict else vlan['ip_address_secundary']  # noqa
                                    except Exception:
                                        ip_secundary_list = []

                                    if ip_secundary_list is not [{}]:

                                        COLUMN = 16
                                        NUM_IP = 2
                                        for secundary in ip_secundary_list:

                                            try:
                                                EC_VLAN_IP_ADDRESS = secundary['ip_address']  # noqa
                                                EC_VLAN_IP_ADDRESS_MASK = secundary['mask']  # noqa
                                                EC_VLAN_IP_ADDRESS_TYPE = secundary['type']  # noqa

                                                if EC_VLAN_IP_ADDRESS is not None:  # noqa
                                                    EC_VLAN_IP_SECUNDARY = f"{EC_VLAN_IP_ADDRESS} {EC_VLAN_IP_ADDRESS_MASK} {EC_VLAN_IP_ADDRESS_TYPE}"  # noqa
                                            except Exception:
                                                EC_VLAN_IP_SECUNDARY = None

                                            self.ws.cell(row=1, column=COLUMN).value = f'IP ADDRESS {NUM_IP}'  # noqa
                                            self.ws.cell(row=index, column=COLUMN).value = EC_VLAN_IP_SECUNDARY  # noqa
                                            COLUMN += 1
                                            NUM_IP += 1

                                    # fin por multiples columnas en excel
                                    index += 1

                        else:
                            # --
                            # -- si el servicio no tiene vlans programdas
                            # datos servicios
                            self.ws[f"A{index}"] = EC_INTERFACE_NAME
                            self.ws[f"E{index}"] = EC_SERVICE_BRIDGE_DOMANIN
                            self.ws[f"F{index}"] = EC_SERVICE_DOT1Q
                            self.ws[f"G{index}"] = EC_SERVICE_DESCRIPTION

                            # --
                            # -- [validacion] - se pega el dato solo si es pw_class # noqa
                            # --
                            self.ws[f"J{index}"] = EC_SERVICE_XCONNECTIONS
                            self.ws[f"K{index}"] = EC_SERVICE_XCONNECTIONS_VC_ID  # noqa
                            self.ws[f"L{index}"] = EC_SERVICE_PW_CLASS_DATA if EC_SERVICE_PW_CLASS == 'pw_class' else None  # noqa

                            self.ws[f"M{index}"] = EC_SERVICE_QOS_INPUT
                            self.ws[f"N{index}"] = EC_SERVICE_QOS_OUTPUT
                            index += 1

    def save(self, name):
        self.wb.save(f"{name}.xlsx")



from typing import Optional

try:
    from .c_typer import app, typer
except Exception:
    from c_typer import app, typer

try:
    from .c_login import Login
except Exception:
    from c_login import Login


def callback_validate_name(value: str):
    if value is None or value == "":
        raise typer.BadParameter(
            "a name has not been defined",
            param_hint="--name")
    return value


def callback_validate_password(value: str):
    if value is None or value == "":
        raise typer.BadParameter(
            "password is required",
            param_hint="--password")
    return value


@app.command()
def login(
    name: Optional[str] = typer.Option(
        ...,
        help="The name user for ciberc-ca",
        callback=callback_validate_name
    ),
    password: str = typer.Option(
        ...,
        prompt=True,
        confirmation_prompt=True,
        hide_input=True,
        callback=callback_validate_password)
):
    """
    Login on (CiberC Code Automations)
    """

    try:
        login = Login()
        login.init(username=name, password=password)
        login.saveToken()

        if login.getToken() is None:
            raise typer.BadParameter(
                "error to login on ciberc-ca")
    except Exception:
        raise typer.BadParameter(
            "error to login on ciberc-ca")

    message = typer.style("login success", fg=typer.colors.GREEN, bold=True)
    typer.echo(message)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cibercca']

package_data = \
{'': ['*'], 'cibercca': ['inventory/*']}

install_requires = \
['PyYAML==6.0',
 'hvac==0.11.2',
 'napalm==3.3.1',
 'nornir==3.2.0',
 'openpyxl==3.0.9',
 'pandas==1.3.4',
 'tabulate==0.8.9',
 'tqdm==4.62.3',
 'ttp==0.8.1',
 'typer==0.4.0']

entry_points = \
{'console_scripts': ['ciberc-ca = cibercca.main:main']}

setup_kwargs = {
    'name': 'cibercca',
    'version': '0.1.0',
    'description': 'CiberC Code Automation - reports excel and json formats',
    'long_description': None,
    'author': 'Rafael Garcia Sagastume',
    'author_email': 'rafael.garcia@ciberc.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



try:
    from .c_typer import app
except Exception:
    from c_typer import app


# tipo de importacion para identificar con cython o manual
try:
    from .c_login_command import app as login  # noqa
except Exception:
    from c_login_command import app as login  # noqa

try:
    from .c_alive_command import app as alive  # noqa
except Exception:
    from c_alive_command import app as alive  # noqa

try:
    from .c_interfaces_command import app as interfaces  # noqa
except Exception:
    from c_interfaces_command import app as interfaces  # noqa

try:
    from .c_inventory_command import app as inventory  # noqa
except Exception:
    from c_inventory_command import app as inventory  # noqa


def main():
    return app()


if __name__ == "__main__":
    main()



from pathlib import Path
from typing import Optional

try:
    from .c_typer import app, typer
except Exception:
    from c_typer import app, typer

try:
    from .c_interfaces import Interfaces
except Exception:
    from c_interfaces import Interfaces


# almacena la informacion del estado para los comandos
state = {}


def callback_interface_output(value: str):  # noqa
    if value not in ['json', 'excel']:
        raise typer.BadParameter("Only json or excel type is valid",
                                 param_hint="--output")
    state['output'] = value
    return value


def callback_interface_path(value: Path):  # noqa
    if value.is_dir():
        return value
    raise typer.BadParameter("inventory is not directory",
                             param_name="--path")


def callback_interface_mechanism(value: str):  # noqa
    if state['output'] == "excel":
        if value is None or value == "":
            raise typer.BadParameter(
                "no print mechanism defined",
                param_hint="--mechanism")
        if value not in ['row', 'column']:
            raise typer.BadParameter("Only row or column mechanisms",
                                     param_hint="--mechanism")
    return value


def callback_interface_name(value: str):  # noqa
    if state['output'] == "excel":
        if value is None or value == "":
            raise typer.BadParameter(
                "a name has not been defined for the excel file",
                param_hint="--name")
    return value


@app.command()
def interfaces(
    path: Optional[Path] = typer.Option(
        ...,
        help="The path to inventory",
        # callback=callback_interface_path
    ),
    group: Optional[str] = typer.Option(
        None,
        help="The groups to filter inventory"
    ),
    workers: Optional[int] = typer.Option(
        2,
        help="The parallel execution"
    ),
    output: Optional[str] = typer.Option(
        "json",
        help="The type to print report",
        # callback=callback_interface_output
    ),
    mechanism: Optional[str] = typer.Option(
        "",
        help="The excel mechanism to print report",
        # callback=callback_interface_mechanism
    ),
    name: Optional[str] = typer.Option(
        "",
        help="The name of excel report",
        # callback=callback_interface_name
    ),
):
    """
    informacion de las interfaces de los dispositivos:

    Example:\n
        - python3 core/main.py interfaces --path=core/inventory/ --output=json > interfaces.json\n
        - python3 core/main.py interfaces --path=core/inventory/ --output=excel --mechanism=row --name=interfaces > interfaces.json
    """  # noqa

    # validaciones callback
    callback_interface_path(path)
    callback_interface_output(output)
    callback_interface_mechanism(mechanism)
    callback_interface_name(name)

    # se ejecuta si todas las validaciones
    # de los argumentos son correctas
    data = Interfaces(path=str(path), filter=group, workers=workers)
    data.run()

    # crea el proceso con estructura json
    if output == 'json':
        print(data.data_json())

    # para el reporte de excel se necesita
    # data, mechanism
    if output == 'excel':
        status = data.data_excel(mechanism, name)
        if status is False:
            return "excel not created."
        print(data.data_json())
